//Author: Chad Carter
//Website: http://xnaessentials.com/

using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace SelectionWheel
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        Texture2D selection;
        Texture2D selector;

        SpriteFont font;

        const int ROUNDED = 0; //start with the selection on the right first 
        const int ELIPSE = 1; //work your way counter-clockwise 
        const int SQUARE = 2;
        const int STAR = 3;
        const int INFINITY = 4;
        const int ABSTRACT = 5;
        const int PLUS = 6;
        const int CLOUD = 7;

        Dictionary<int, string> selections = new Dictionary<int, string>();

        float radians = 0;
        float degrees = 0;
        int index;
        int index2;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);

            //Will be scaled down if not supported
            graphics.PreferredBackBufferWidth = 1920;
            graphics.PreferredBackBufferHeight = 1080;

            graphics.ApplyChanges();

            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            InitializeSelections();

            base.Initialize();
        }

        void InitializeSelections()
        {
            selections = new Dictionary<int, string>();
            selections.Add(ROUNDED, "ROUNDED");
            selections.Add(ELIPSE, "ELIPSE");
            selections.Add(SQUARE, "SQUARE");
            selections.Add(STAR, "STAR");
            selections.Add(INFINITY, "INFINITY");
            selections.Add(ABSTRACT, "ABSTRACT");
            selections.Add(PLUS, "PLUS");
            selections.Add(CLOUD, "CLOUD");

            index = 0; // default to selecting the value on the right (ROUNDED)
        }


        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            selection = Content.Load<Texture2D>("selection");
            selector = Content.Load<Texture2D>("selector");

            font = Content.Load<SpriteFont>("font");
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            GamePadState gp = GamePad.GetState(PlayerIndex.One);

            // Allows the game to exit
            if (gp.Buttons.Back == ButtonState.Pressed || Keyboard.GetState().IsKeyDown(Keys.Escape))
                this.Exit();

            //Get radians from thumbstick
            if (gp.ThumbSticks.Left.Y != 0 || gp.ThumbSticks.Left.X != 0)
                radians = (float)(Math.Atan2(gp.ThumbSticks.Left.Y, gp.ThumbSticks.Left.X));

            //convert to angle from radians to degrees
            degrees = radians * 180.0f / (float)Math.PI;

            //changes -90 to 270
            if (degrees < 0) //Atan returns negative for quadrant 3 
                degrees += 360; //so make it positive by adding 360 to it (get values 270-360); 

            //to make the above dynamic we could divide 360 by the number of items in our wheel (360/8 = 45)
            //and the offset would need to be that value (45) / 2 = 22.5
            //float itemDegree = 360.0f/selections.Count;
            //float offset = itemDegree / 2.0f;
            //index = (int)((degrees + offset) / itemDegree); //We have 8 selection points. 8 goes into 360 a total of 45 times.

            //We have 8 selection points. 8 goes into 360 a total of 45 times.
            index = (int)((degrees + 22.5f) / 45);

            if (index == selections.Count)
                index = 0; //since we are adding in 22.5, we will go over, so reset it

            //A little faster, more concise, less code, less readable way is to
            index2 = (int)((radians + MathHelper.TwoPi + (0.3926990816987241f)) / MathHelper.PiOver4) & (selections.Count - 1);
            //radians + MathHelper.TwoPi is to get a positive value since 270 degrees is -1.56761101961531 then 4.71238898038469
            //0.3926990816987241 = MathHelper.PiOver4 / 2 (22.5 degrees) (half of 45 degrees or PiOver4 - 3.14/4)
            //we get our index by bit ANDing our direction count 8 (0-7) so 7
            
            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here
            spriteBatch.Begin();
            spriteBatch.DrawString(font, selections[index], new Vector2(530, 0), Color.Red);
            spriteBatch.DrawString(font, selections[index2], new Vector2(530, 50), Color.Red);

            //To see the radians and degrees in action, uncomment this code
            //spriteBatch.DrawString(font, "Radians: " + radians.ToString(), new Vector2(530, 100), Color.Red);
            //spriteBatch.DrawString(font, "Degrees: " + degrees.ToString(), new Vector2(530, 150), Color.Red);

            spriteBatch.Draw(selection, new Rectangle(0, 0, 512, 512), Color.White);
            spriteBatch.Draw(selector, new Rectangle(256, 256, 512, 512), null, Color.White,
                                    -radians, new Vector2(256, 256), SpriteEffects.None, 0);
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
